@@ -65,42 +65,31 @@ FuzzyBinomialNaiveBayes.default <- function(train, cl, cores = 2, fuzzy = T) {
   # Estimating class parameters
   p_data <- predata(train,cl)
   # --
-  train <-  p_data$train
+  dados <- train <-  p_data$train
   cols <- p_data$cols
-  dados <- p_data$dados
   M <- p_data$M
   intervalos <- p_data$intervalos
   # --------------------------------------------------------
   # --------------------------------------------------------
-  # Verify data types
-  verifyNumbers <- sapply(1:cols, function(i){
-    n = 3
-    subset <- sample(dados[,i],size = n, replace = F)
-    result <- subset == floor(subset)
-    if(sum(result) == n){
-      result <- 1
-    }else{
-      result <- 0
-    }
-    return(result)
-  })
+  # Verify if all variables are discrete
+  verifyNumbers <- verifyNumbersFunction(dados, cols)
 
   # --------------------------------------------------------
   if(sum(verifyNumbers) != cols){ stop("All variables must be discrete values.") }
   # --------------------------------------------------------
 
 
-
   #--------------------------------------------------------
   # Estimating Gamma Parameters
   parametersC <- estimation_parameters_binomial(M, cols, dados)
   #--------------------------------------------------------
   #--------------------------------------------------------
-  Sturges <- Sturges(dados, M);
-  Comprim_Intervalo <- Comprim_Intervalo(dados, M, Sturges);
+  Sturges <- Sturges_discrete(dados, M);
+  Comprim_Intervalo <- Comprim_Intervalo_discrete(dados, M);
   minimos <- minimos(dados, M, cols);
-  Freq <- Freq(dados, M, Comprim_Intervalo, Sturges, minimos, cols);
-  Pertinencia <- Pertinencia(Freq, dados, M);
+  MinimosDataFrame <- minomosdt_function(minimos, M, Comprim_Intervalo, Sturges, cols)
+  Frequencia <- Freq_esparsa(dados = dados,M = M, minomosdt = MinimosDataFrame, cols = cols)
+  Pertinencia <- Pertinencia_esparsa(M = M, Frequencia, cols = cols)
   #------
   # A priori probability of classes - considered equal
   pk <- rep(1 / length(unique(M)), length(unique(M)))
@@ -111,6 +100,7 @@ FuzzyBinomialNaiveBayes.default <- function(train, cl, cores = 2, fuzzy = T) {
   structure(list(
     parametersC = parametersC,
     minimos = minimos,
+    MinimosDataFrame = MinimosDataFrame,
     cols = cols,
     M = M,
     cores = cores,
@@ -152,6 +142,7 @@ predict.FuzzyBinomialNaiveBayes <- function(object,
   #--------------------------------------------------------
   parametersC <- object$parametersC
   minimos <- object$minimos
+  MinimosDataFrame <-  object$MinimosDataFrame
   cols <- object$cols
   M <- object$M
   cores <- object$cores
@@ -169,11 +160,10 @@ predict.FuzzyBinomialNaiveBayes <- function(object,
   # ---------
   N_test <- nrow(test)
   # --
-  test <- split(test, seq(nrow(test)))
   # --
   if(fuzzy == T){
-    retorno <- purrr::map(test, function_membership_predict, M, Sturges, minimos, Comprim_Intervalo, Pertinencia, cols)
-    R_M_obs <- function_fuzzy_predict(retorno, P, M)
+    Pertinencia_r <- function_new_membership_predict(test, M = M, MinimosDataFrame, Pertinencia, cols = cols)
+    R_M_obs <- function_new_fuzzy_predict(retorno = Pertinencia_r, P, M)
   }else{
     R_M_obs <- t(data.frame(matrix(unlist(P), nrow=length(P), byrow=TRUE)))
   }
